#ifndef DataSourceProcessor_h
#define DataSourceProcessor_h 1

#include "luiza/Processor.h"

using namespace gloria ;


namespace luiza{
  
  //! Base class for processors accessing input data
  /*! Base class for data source handlers that can read arbitrary input files (or net) and 
   *  create data collections to be processed by other processors. When all data sources 
   *  are read and closed data processing finishes. 
   *
   *  @author A.F.Zarnecki, University of Warsaw
   */
  
  class DataSourceProcessor : public Processor {
    
  public:
    
    DataSourceProcessor( const std::string& typeName ) ;


    /** User call back to read the data source and create the LCIO event and run header.
     *  The user needs to call ProcessMgr::processRunHeader(LCRunHeader* rhdr) and 
     *  ProcessMgr::processEvent(LCEvent* evt) in order to call the relevant Processor methods in 
     *  readDataSource().<br>
     *  \param numEvents The number of events the DataSourceProcessor should generate.
     */

    inline bool OpenDataSource() { return _isInputInitialized && (! _isInputClosed); };

  protected:

    bool _isInputInitialized;
    bool _isInputClosed;
  };
 
} // end namespace marlin 

#endif
