// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef LuizaExceptions_h
#define LuizaExceptions_h 1

#include "Processor.h" 
#include "Exceptions.h" 

// define some exceptions for Luiza

namespace luiza{

  //! ParseException used for parse errors, e.g. when reading the steering file.

  class ParseException : public Exception{
    
  protected:
    ParseException() {  /*no_op*/ ; } 
  public: 
    virtual ~ParseException() throw() { /*no_op*/; } 

    ParseException( std::string text ){
      message = "luiza::ParseException: " + text ;
    }
  }; 


  //! SkipEventException used to skip the current event in Processor::processData
  /*! By using SkipEventException processor forcess luzia::ProcessorMgr
   *  to interupt processing of current data loop and start next
   *  loop. This can be useful eg. when the astrometry failed on
   *  particular frame (so the following analysis will be useless anyway)
   */
  class SkipEventException : public Exception{

  protected:
    SkipEventException() {  /*no_op*/ ; } 

  public: 
    SkipEventException(const Processor* proc){
      message = proc->name()  ;
    }
    virtual ~SkipEventException() throw() { /*no_op*/; } 

  }; 

  //! StopProcessingException used to stop proccessing of data
  /*! By using StopProcessingException processor forces luiza::ProcessorMgr
   *  to conclude analysis after the current loop.
   *  Processor::concludeAnalysis() and Processor::end() methods are
   *  called for each active processor, as if the requested number of
   *  events is reached.
   */
  class StopProcessingException : public Exception{

  protected:
    StopProcessingException() {  /*no_op*/ ; } 

  public: 
    StopProcessingException(const Processor* proc){
      message = proc->name()  ;
    }
    virtual ~StopProcessingException() throw() { /*no_op*/; } 

  }; 

    
  //! AbortProcessingException used to abort proccessing of data
  /*! By using AbortProcessingException processor forces luiza::ProcessorMgr
   *  to abort analysis at given point. Current loop is interupted,
   *  Processor::concludeAnalysis() methods are NOT called (only
   *  Processor:end()). This should be used only in case of fatal
   *  errors, when all results are useless (eg. no connection to the
   *  object data base). 
   */
  class AbortProcessingException : public Exception{

  protected:
    AbortProcessingException() {  /*no_op*/ ; } 

  public: 
    AbortProcessingException(const Processor* proc){
      message = proc->name()  ;
    }
    virtual ~AbortProcessingException() throw() { /*no_op*/; } 

  }; 

    

} // end namespace 

#endif



