// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef TestProcessor_h
#define TestProcessor_h 1

#include "Processor.h"

using namespace gloria;

namespace luiza {
  
  
  //! Simple processor for testing data collection. 
  /*! Writes something to stdout 
   *  for every callback method.
   *  
   *  Also checks sizes of all images stored in given image collection
   *  (or in all collections within GloriaDataContainer) and dumps
   *  this information for every loop (DEBUG level).
   *
   *  Prints summary table with sizes of all images at the end of the processing.
   */

  class TestProcessor : public Processor {
    
  public:
    
    virtual Processor*  newProcessor() { return new TestProcessor ; }
    
    
    TestProcessor() ;
    
    //! Called at the begin of the job before anything is read.
    /*! Use to initialize the processor, e.g. book histograms.
     */
    virtual void init() ;
    
    //! Called before first data loop.
    /*! Can be used to create permanent data structures, if needed.
     */
    virtual void startAnalysis( GloriaDataContainer* data ) ;
    
 
    //! Called for every data collection

    virtual void processData( GloriaDataContainer* data ) ;
    
    
    virtual void check( GloriaDataContainer* data ) ; 
    
    //! Called after all data processing loops to finalize the analysis

    virtual void concludeAnalysis( GloriaDataContainer* ) ;
    
    
    //! Called after data processing for clean up.

    virtual void end() ;
    
    
  protected:

    /**Test method for const.*/
    void printEndMessage() const ;

    string _imageCollectionName;
    string _tableCollectionName;

    int _nLoops ;
    int _nImgCollections ;
    int _nImages ;
    int _nTblCollections ;
    int _nTables ;

    GloriaFitsTable *_imgTable;
    GloriaFitsTable *_tblTable;

  } ;
  
} // end namespace luiza
#endif



