// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

// Author: A.F.Zarnecki, University of Warsaw <mailto:zarnecki@fuw.edu.pl>

/*
 *   This source code is part of the LUIZA software package for GLORIA.
 *   You are free to use this source files for your own development as
 *   long as it stays in a public research context. You are not
 *   allowed to use it for commercial purpose. You must put this
 *   header with author names in all development based on this file.
 *
 */

#include "gloria/GloriaTableColumn.h"


using namespace std;

namespace gloria{
  
template<class T>
TableColumnType GetValueType(T val) { return UndefinedColumn; };

template<>
TableColumnType GetValueType(long val) { return LongColumn; };

template<>
TableColumnType GetValueType(double val) { return DoubleColumn; };

template<>
TableColumnType GetValueType(string val) { return StringColumn; };

template<>
TableColumnType GetValueType(LongVec val) { return LongVectorColumn; };

template<>
TableColumnType GetValueType(DoubleVec val) { return DoubleVectorColumn; };



template<class T>
string GetValueTypeName(T val) { return string("Undefined"); };

template<>
string GetValueTypeName(long val) { return string("Long"); };

template<>
string GetValueTypeName(double val) { return string("Double"); };

template<>
string GetValueTypeName(string val) { return string("String"); };

template<>
string GetValueTypeName(LongVec val) { return string("Vector of Longs"); };

template<>
string GetValueTypeName(DoubleVec val) { return string("Vector of Doubles"); };



string GloriaTableColumn::TypeName()
  {
    string tnames[] = {"Undefined","Long","Double","String","Vector of Longs","Vector of Doubles"};

    return tnames[_type];
  }


template <class T>
int  GloriaTableColumnOf<T>::ValSize(int i) 
   { 
   if(i>= _data.size() || i<0 )return 0;

   int siz=1;
   if( _type == LongVectorColumn) siz = ((LongVecVec *) &_data)->at(i).size(); 
   if( _type == DoubleVectorColumn) siz = ((DoubleVecVec *) &_data)->at(i).size(); 
   return siz; 
   } 


template <class T>
long GloriaTableColumnOf<T>::LongVal(int i, int j)  
   { 
   if(i>= _data.size() || i<0 )return 0;

   long val=0 ; 
   if( _type == LongColumn) val= ((LongVec *) &_data)->at(i); 
   if( _type == LongVectorColumn) val= ((LongVecVec *) &_data)->at(i)[j]; 

   return val; 
   }

template <class T>
double GloriaTableColumnOf<T>::DoubleVal(int i, int j) 
   {
   if(i>= _data.size() || i<0 )return 0.;

   double val=0. ; 
   if( _type == DoubleColumn) val= ((DoubleVec *) &_data)->at(i); 
   if( _type == LongColumn) val= (double) ((LongVec *) &_data)->at(i); 
   if( _type == DoubleVectorColumn) val= ((DoubleVecVec *) &_data)->at(i)[j]; 
   if( _type == LongVectorColumn) val= (double) ((LongVecVec *) &_data)->at(i)[j]; 
   return val; 
   }



template <class T>
string GloriaTableColumnOf<T>::StringVal(int i)  
  { 
  if(i>= _data.size() || i<0 )return string("");

  stringstream val; 

  if( _type == DoubleColumn) val << std::fixed << ((DoubleVec *) &_data)->at(i); 
  if( _type == LongColumn) val << ((LongVec *) &_data)->at(i); 
  if( _type == StringColumn) val << ((StringVec *) &_data)->at(i); 

  if( _type == DoubleVectorColumn)
      { 
      int vsize = ((DoubleVecVec *) &_data)->at(i).size();
      for(int j=0;j<vsize;j++) 
            val <<  ((DoubleVecVec *) &_data)->at(i)[j] << " "; 
      }

  if( _type == LongVectorColumn) 
      { 
      int vsize = ((LongVecVec *) &_data)->at(i).size();
      for(int j=0;j<vsize;j++) 
            val << ((LongVecVec *) &_data)->at(i)[j] << " ";
      }
 
  //  if(_unit.size()) val << _unit ;

  return val.str(); 
  }


template <class T>
string GloriaTableColumnOf<T>::StringVal(int i, int j)  
  { 
  if(i>= _data.size() || i<0 )return string("");

  stringstream val; 

  if( _type == DoubleColumn) val << ((DoubleVec *) &_data)->at(i); 
  if( _type == LongColumn) val << ((LongVec *) &_data)->at(i); 
  if( _type == StringColumn) val << ((StringVec *) &_data)->at(i); 
  if( _type == DoubleVectorColumn) val <<  ((DoubleVecVec *) &_data)->at(i)[j]; 
  if( _type == LongVectorColumn)  val << ((LongVecVec *) &_data)->at(i)[j];
 
  //  if(_unit.size()) val << _unit ;

  return val.str(); 
  }


template <class T>
GloriaTableColumn* GloriaTableColumnOf<T>::Clone()
  {
   GloriaTableColumnOf<T>* clone =  new GloriaTableColumnOf<T>(_name, _emptyVal, _unit, _comment);

   clone->_data = _data;

   return (GloriaTableColumn*) clone;
  }
  

template <class T>
GloriaTableColumn* GloriaTableColumnOf<T>::EmptyClone()
  {
   GloriaTableColumnOf<T>* clone =  new GloriaTableColumnOf<T>(_name, _emptyVal, _unit, _comment);
   return (GloriaTableColumn*) clone;
  }
  

template <class T>
void GloriaTableColumnOf<T>::CopyCellValue(GloriaTableColumn* source, int isource, int itarget)
{
  GloriaTableColumnOf<T>* tsource = (GloriaTableColumnOf<T>* )source;
  
  // Just to be sure
  if(tsource == NULL)return;
  if(tsource->Type() != _type)return;

  if(isource<0 || isource>=tsource->Size() )return;

  T val = tsource->At(isource);

  if(itarget >= 0 && itarget < _data.size()) 
    _data.at(itarget)=val;
  else
    _data.push_back(val);
}

template <class T>
long* GloriaTableColumnOf<T>::CreateSortIndex(bool reverse)
{
  long* index = NULL;

  // Can not create index for vector columns - return NULL pointer
  
  if( _type == LongVectorColumn ||
      _type == DoubleVectorColumn ||
      _type == UndefinedColumn )   return index;

  long nRow = _data.size();

  // Empty column: return NULL pointer

  if(nRow == 0) return index;

  std::multimap< T ,long> columnMap;

  // 'typename' required by compiler for template iterator !
  typename std::multimap< T ,long>::iterator columnIterator;

  for(long irow=0; irow<nRow;irow++)
    columnMap.insert(make_pair( _data.at(irow), irow ) );
 
  index = new long[nRow];

  long irow=(reverse)? nRow-1 : 0 ;
  long step=(reverse)?     -1 : 1 ;

  for(columnIterator=columnMap.begin(); columnIterator!=columnMap.end(); columnIterator++)
    {
      index[irow] = columnIterator->second;
      irow+=step;
    }

  return index;
}

template <class T>
void GloriaTableColumnOf<T>::SortByIndex(long* index)
{
  if(index == NULL) return;

  long nRow = _data.size();

  if(nRow == 0) return;

  T* newColumn = new T[nRow];

  for(long irow=0; irow<nRow; irow++)
    newColumn[irow] = _data.at(index[irow]);

  for(long irow=0; irow<nRow; irow++)
    _data.at(irow) = newColumn[irow];

  delete [] newColumn;

  return;
}

// Needed to avoid linker errors
// see: http://www.parashift.com/c++-faq-lite/templates.html#faq-35.15

template class GloriaTableColumnOf<long>; 
template class GloriaTableColumnOf<double>; 
template class GloriaTableColumnOf<string>; 
template class GloriaTableColumnOf<LongVec>; 
template class GloriaTableColumnOf<DoubleVec>; 
}
