// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

// Author: A.F.Zarnecki, University of Warsaw <mailto:zarnecki@fuw.edu.pl>

/*
 *   This source code is part of the LUIZA software package for GLORIA.
 *   You are free to use this source files for your own development as
 *   long as it stays in a public research context. You are not
 *   allowed to use it for commercial purpose. You must put this
 *   header with author names in all development based on this file.
 *
 */

#include "gloria/GloriaTelescope.h"

namespace gloria{

// Default constructor

GloriaTelescope::GloriaTelescope()
{
_id = 0;
_name.clear();
_longitude = _latitude = _elevation = 0.;

 _parameters = new luiza::StringParameters();
}


// Copy constructor

GloriaTelescope::GloriaTelescope(const GloriaTelescope &t)
{
_id = t._id;
_name= t._name;
 _longitude = t._longitude;
 _latitude = t._latitude;
 _elevation = t._elevation;

 _parameters = new luiza::StringParameters(*t._parameters);
}


// Named constructor

GloriaTelescope::GloriaTelescope(int id, std::string name)
{
_id = id;
_name.push_back(name);
_longitude = _latitude = _elevation = 0.;

 _parameters = new luiza::StringParameters();
}


// Default destructor

GloriaTelescope::~GloriaTelescope()
{
delete _parameters;
}


// Add parameter

void GloriaTelescope::AddParameter(std::string key, std::string value)
{
  if(key.size() == 0) return;

  std::vector<std::string> tmpVec;

  tmpVec.push_back(key);
  tmpVec.push_back(value);

  _parameters->add(tmpVec);

}
      
std::ostream& operator<< (  std::ostream& s,  GloriaTelescope& t ) 
{
  s << " Telescope ID = " << t._id ;

  if(t._name.size()>0)
    s << " : " << t._name[0];
 
  for(int in=1; in<t._name.size(); in++)
    s << ", " << t._name[in]  ;

  s << std::endl ;

  s << " Longitude = " << t._longitude << "  Latitude = " << t._latitude; 
  s << " Elevation = " << t._elevation << std::endl;
  s << " Parameters " << std::endl;

  s << *t._parameters << std::endl;

  return s ;
}

} // namespace gloria
