//#include "lcio.h"

//#ifdef LCIO_MAJOR_VERSION 
//#if LCIO_VERSION_GE( 1,2)  
//#include "LCIOSTLTypes.h"
//#endif
//#else
//#include "MarlinLCIOSTLTypes.h"
//#endif

#include "luiza/LuizaSTLTypes.h"


#include "luiza/ProcessorMgr.h"
#include "luiza/Processor.h"
//#include "marlin/ProcessorEventSeeder.h"
//#include "marlin/Exceptions.h"
//#include "IO/LCReader.h"

#include "luiza/Parser.h"
#include "luiza/XMLParser.h"

#include "luiza/Global.h"

#include "gloria/GloriaNetworkMgr.h"

#include "luiza/LuizaSteerCheck.h"

//#include "marlin/XMLFixCollTypes.h"

#include <sstream>
#include <fstream>
#include <string>
#include <assert.h>


#include <cstring>
#include <algorithm>


#include "luiza/ProcessorLoader.h"

#include "luiza/VerbosityLevels.h"
#include "streamlog/streamlog.h"

using namespace std ;
using namespace gloria ;
using namespace luiza ;


void createProcessors( Parser&  parser ) ;
// void createProcessors( XMLParser&  parser ) ;
void  createProcessors( const IParser&  parser) ;

void listAvailableProcessors() ;
void listAvailableProcessorsXML() ;
int printUsage() ;


/** LCIO framework that can be used to analyse LCIO data files
 *  in a modular way. All tasks have to be implemented in Subclasses
 *  of Processor. They will be called in the order specified in the steering file.
 *
 */
int main(int argc, char** argv ){

  // this macro enables printout of uncaught exceptions
  //    HANDLE_LCIO_EXCEPTIONS


    //###### init streamlog ######

    std::string binname = argv[0]  ;
    binname = binname.substr( binname.find_last_of("/") + 1 , binname.size() ) ;
    streamlog::out.init( std::cout , binname ) ;


    if( argc > 1 ){
        if( std::string(argv[1]) == "-x" ){
            std::cout  << "<?xml version=\"1.0\" encoding=\"us-ascii\"?>" << std::endl
                << "<!-- ?xml-stylesheet type=\"text/xsl\" href=\"http://ilcsoft.desy.de/marlin/marlin.xsl\"? -->" << std::endl
                << "<!-- ?xml-stylesheet type=\"text/xsl\" href=\"marlin.xsl\"? -->" << std::endl << std::endl;
        }
    }

    //#ifndef LUIZA_NO_DLL

    //------ load shared libraries with processors ------

    luiza::StringVec libs ;
    LCTokenizer t( libs, ':' ) ;

    std::string luizaProcs("") ;

    char * var =  getenv("LUIZA_DLL" ) ;

    if( var != 0 ) {
        luizaProcs = var ;
    } else {
        std::cout << std::endl << "<!-- You have no LUIZA_DLL variable in your environment "
            " - so no processors will be loaded. ! --> " << std::endl << std::endl ;
    }

    std::for_each( luizaProcs.begin(), luizaProcs.end(), t ) ;

    ProcessorLoader loader( libs.begin() , libs.end()  ) ;
    if( loader.failedLoading() ){
        return(1);
    }

    //------- end processor libs -------------------------

    //#endif


    //------- parse command line for arguments and steering file name ---------------

    const char* steeringFileName = "none"  ;

    //map<string, map<string,string> > cmdlineparams; 

    CommandLineParametersMap cmdlineparams; 

    // check for dynamic command line arguments first

    for( int i = 1 ; i < argc ; i++ ) {
        // cout << "argv[" << i << "]:\t" << argv[i] << endl ;

        if( string( argv[i] ).substr( 0, 2 ) == "--" ){
            // cout << "dynamic opt:\t" << string( argv[i] ).substr( 2 ) << endl ;

            // split dynamic argument by '=', i.e.
            // --global.LCIOInputFiles="1.slcio 2.slcio 3.slcio" --> global.LCIOInputFiles , "1.slcio 2.slcio 3.slcio"
            luiza::StringVec cmdlinearg, cmdlinekey ;
            LCTokenizer t( cmdlinearg, '=' ) ;

            string param( argv[i] ) ;

            string s( param.substr( 2 ) ) ;
            for_each( s.begin(), s.end(), t ) ;

            // cout << "split opt:\tkey: " << cmdlinearg[0] << ", value: " << cmdlinearg[1] << endl ;

            if( cmdlinearg.size() != 2 ){
                cerr << endl << "*** invalid command line option: " << argv[i] << endl << endl;
                return printUsage();
            }

            // split first arg by '.'
            // --global.LCIOInputFiles --> global , LCIOInputFiles
            s = cmdlinearg[0] ;
            LCTokenizer t2( cmdlinekey, '.' ) ;

            for_each( s.begin(), s.end(), t2 ) ;

            if( cmdlinekey.size() != 2 ){
                cerr << endl << "*** invalid command line option: " << argv[i] << endl << endl;
                return printUsage();
            }

            // case insensitive command line options
            std::transform(cmdlinekey[0].begin(), cmdlinekey[0].end(), cmdlinekey[0].begin(), ::toupper);
            std::transform(cmdlinekey[1].begin(), cmdlinekey[1].end(), cmdlinekey[1].begin(), ::toupper);

            // cout << "split key:\tindex1: " << cmdlinekey[0] << ", index2: " << cmdlinekey[1] << endl ;

            // save dynamic options into map
            cmdlineparams[ cmdlinekey[0] ][ cmdlinekey[1] ] = cmdlinearg[1] ;

            cout << "<!-- steering file parameter: [ " << cmdlinekey[0] << "." << cmdlinekey[1] << " ] will be OVERWRITTEN with value: [\"" << cmdlinearg[1] << "\"] -->" << endl;

            // erase dynamic options from **argv
            for( int j = i ; j < argc-1 ; j++ ){
                argv[j] = argv[j+1];
            }
            argc--;
            i--;
        }
    }

    cout << endl ;

    // read file name from command line

    if( argc > 1 ){

        if( std::string(argv[1]) == "-l" ){
            listAvailableProcessors() ;
            return(0) ;
        }
        else if( std::string(argv[1]) == "-x" ){
            listAvailableProcessorsXML() ;
            return(0) ;
        }
       else if( std::string(argv[1]) == "-c" ){
            if( argc == 3 ){
                LuizaSteerCheck msc(argv[2], &cmdlineparams );
                msc.dump_information();
                return(0) ;
            }
            else{
                std::cout << "  usage: Luiza -c steeringFile.xml" << std::endl << std::endl;
                return(1);
            }
	} 
/*        else if( std::string(argv[1]) == "-d" ){
            if( argc == 4 ){
                LuizaSteerCheck msc(argv[2], &cmdlineparams );
                msc.saveAsDOTFile(argv[3]);
                return(0) ;
            }
            else{
                std::cout << "  usage: Luiza -d steer.xml diagram.dot" << std::endl << std::endl;
                return(1);
            }
        }
*/
        else if( std::string(argv[1]) == "-h"  || std::string(argv[1]) == "-?" ){

            return printUsage() ;
        }


        // one argument given: the steering file for normal running :
        steeringFileName = argv[1] ;

    } else {

        return printUsage() ;
    }


    // --------------------- end of parsing command line ------------------------------


    // --------------- Parse input steering file and initialize processors ------------

    IParser* parser ;

    // assume  xml  steering file

    std::string filen(  steeringFileName ) ;

    parser = new XMLParser( steeringFileName ) ;

    // tell parser to take into account any options defined on the command line
    parser->setCmdLineParameters( cmdlineparams ) ;

    parser->parse() ;

    Global::parameters = parser->getParameters("Global")  ;

    //fg: can't use assert, as this generates no code when compiled with NDEBUG
    if( Global::parameters  == 0 ) {
        std::cout << "  Could not get global parameters from steering file ! " << std::endl  
            << "   The program has to exit - sorry ! " 
            << std::endl ;
        return(1) ;
    }

    //-----  register log level names with the logstream ---------
    // Only main levels are introduced

    streamlog::out.addLevelName<DEBUG>() ;
    streamlog::out.addLevelName<MESSAGE>() ;
    streamlog::out.addLevelName<WARNING>() ;
    streamlog::out.addLevelName<ERROR>() ;
    streamlog::out.addLevelName<SILENT>() ;

    //-------- init logging level ------------
    std::string verbosity = Global::parameters->getStringVal("Verbosity" ) ;
    streamlog::logscope scope( streamlog::out ) ;

    scope.setLevel( verbosity ) ;

    // Now we are ready to create processors

    createProcessors( *parser ) ;

    // Try to load network configuration if available

    std::string configFile = Global::parameters->getStringVal("NetworkXMLFile" ) ;

    if( configFile.size() > 0 ) {

       Global::Network = new GloriaNetworkMgr ;

       Global::Network -> LoadConfigXML(configFile);

       streamlog_out( MESSAGE )  << " ---- network configuration instantiated from file  " << configFile  
                                 << std::endl  << *Global::Network << std::endl ;
    
    } else {
    
        streamlog_out( WARNING ) << " ---- no network configuration XML file given  --------- " << std::endl ;
    }


    // -------------- End of steering file parsing -----------------------------------

    delete parser;

    // --------------- Use Processor Manager to do the rest -------------------------

    int maxRecord = Global::parameters->getIntVal("MaxRecordNumber");

    ProcessorMgr::instance()->init() ; 

    try{
        ProcessorMgr::instance()->processingLoop(maxRecord) ; 

    } catch( AbortProcessingException& e){

        streamlog_out(MESSAGE)  << " --------------------------------------------------------- " << std::endl << std::endl 
	                        << " AbortProcessingException thrown from " << e.what() << std::endl << std::endl
                                << " --------------------------------------------------------- " << std::endl;
        }  

    ProcessorMgr::instance()->end() ; 


    // Return is causing segmentation fault with database processor based on SOCI ?!
    // That is why exit(0) is used

    //   return 0 ;
    exit(0);
}


//   void  createProcessors(XMLParser&  parser) {
void createProcessors( const IParser&  parser) {

    luiza::StringVec activeProcessors ;
    Global::parameters->getStringVals("ActiveProcessors" , activeProcessors ) ;

    luiza::StringVec procConds ;
    Global::parameters->getStringVals("ProcessorConditions" , procConds ) ;

    bool useConditions = ( activeProcessors.size() == procConds.size() ) ;

    //     for( luiza::StringVec::iterator m = activeProcessors.begin() ; m != activeProcessors.end() ; m++){
    for(unsigned int i=0 ; i<  activeProcessors.size() ; i++ ) {

        StringParameters* p = parser.getParameters( activeProcessors[i] )  ;

        if( p!=0 ){
            std::string type = p->getStringVal("ProcessorType") ;

            if( useConditions ) 
                ProcessorMgr::instance()->addActiveProcessor( type , activeProcessors[i] , p , procConds[i] )  ;
            else
                ProcessorMgr::instance()->addActiveProcessor( type , activeProcessors[i] , p )  ;

        } else{
            std::stringstream sstr ;
            sstr << "Undefined processor : " << activeProcessors[i] << std::endl ;
            streamlog_out( ERROR )  << sstr.str() ;	
            throw Exception( sstr.str() );
        }
    }
}

void  createProcessors(Parser&  parser) {

    luiza::StringVec activeProcessors ;
    Global::parameters->getStringVals("ActiveProcessors" , activeProcessors ) ;

    //for( luiza::StringVec::iterator m = activeProcessors.begin() ; m != activeProcessors.end() ; m++){
    for(unsigned int i=0 ; i<  activeProcessors.size() ; i++ ) {

        //StringParameters* p = parser.getParameters( *m )  ;
        StringParameters* p = parser.getParameters( activeProcessors[i] )  ;


        streamlog_out( MESSAGE ) << " Parameters for processor " << activeProcessors[i]
            << std::endl 
            << *p ; 

        if( p!=0 ){
            std::string type = p->getStringVal("ProcessorType") ;

            //if( ProcessorMgr::instance()->addActiveProcessor( type , *m , p )  ){
            if( ProcessorMgr::instance()->addActiveProcessor( type , activeProcessors[i] , p )  ){

                // 	Processor* processor =  ProcessorMgr::instance()->getActiveProcessor( *m ) ;
                //	processor->setParameters( p ) ;
            }
        } else{
            std::stringstream sstr ;
            sstr << "Undefined processor : " << activeProcessors[i] << std::endl ;
            streamlog_out( ERROR )  << sstr.str() ;	
            throw Exception( sstr.str() );

        }

    }
}

void listAvailableProcessors() {

    ProcessorMgr::instance()->dumpRegisteredProcessors() ;
}

void listAvailableProcessorsXML() {

    ProcessorMgr::instance()->dumpRegisteredProcessorsXML() ;
}


int printUsage() {

    std::cout << " Usage: Luiza [OPTION] [FILE]..." << std::endl 
        << "   runs a Luiza application " << std::endl 
        << std::endl 
        << " Running the application with a given steering file:" << std::endl 
        << "   Luiza steer.xml   " << std::endl 
        << std::endl 
        << "   Luiza -h                  \t print this help information" << std::endl 
        << "   Luiza -?                  \t print this help information" << std::endl 
        << "   Luiza -x                  \t print an example steering file to stdout" << std::endl 
        << "   Luiza -c steer.xml        \t check the given steering file for consistency" << std::endl 
        << "   Luiza -l                  \t list available processors and their parameters" << std::endl 
      //        << "   Luiza -d steer.xml flow.dot\t create a program flow diagram (see: http://www.graphviz.org)" << std::endl 
        << std::endl 
        << " Example: " << std::endl 
        << " To create a new default steering file from any Luiza application, run" << std::endl 
        << "     Luiza -x > mysteer.xml" << std::endl 
        << " and then use either an editor or the LuizaGUI to modify the created steering file " << std::endl 
        << " to configure your application and then run it. e.g. : " << std::endl 
        << "     Luiza mysteer.xml > luiza.out 2>&1 &" << std::endl << std::endl
        << " Dynamic command line options may be specified in order to overwrite individual steering file parameters, e.g.:" << std::endl 
        << "     Luiza --global.LCIOInputFiles=\"input1.slcio input2.slcio\" --global.GearXMLFile=mydetector.xml" << std::endl 
        << "            --MyLCIOOutputProcessor.LCIOWriteMode=WRITE_APPEND --MyLCIOOutputProcessor.LCIOOutputFile=out.slcio steer.xml" << std::endl << std::endl
        << "     NOTE: Dynamic options do NOT work together with Luiza options (-x, -l) nor with the LuizaGUI" << std::endl 
        << std::endl ;

    return(0) ;

}

