// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef ImageMedian_h
#define ImageMedian_h 1

#include "luiza/Processor.h"

using namespace gloria;

namespace luiza {
  
  
  //! Processor to calculate median image from images in the input collection.
  /*! Processor to calculate median image from images in the input collection.
   *  No corrections for possible shifts or any other corrections are
   *  applied. 
   *
   *  \par Input 
   *  Processor takes all images in the specified Gloria data collection and
   *  calculates median value for each pixel.
   *
   *  \param InputCollectionName -
   *             Name of the collection containing input images
   *
   *  \param MatchKeywords
   *             Names for header keywords which should match for
   *             considered frames. If different keyword values are
   *             found in the input collection, multiple median frames
   *             are created, separately for each keyword combination.
   *
   *  \par Output
   *
   *  Resulting median frames are stored in the output collection
   *
   *  \param OutputCollectionName - 
   *            Name of the collection where median image will be stored
   * 
   *  \param PermanentCollection  -
   *   Flag output collection, as permanent, not deleted after loop is
   *   finished (default is false)
   *
   */

  class ImageMedian : public Processor {
    
  public:
    
    virtual Processor*  newProcessor() { return new ImageMedian ; }
    
    
    ImageMedian() ;
    
    /*! Called at the begin of the job before anything is read.
     * Use to initialize the processor, e.g. book histograms.
     */
    virtual void init() ;
    
    /*! Called for every data loop
     */
    virtual void processData( GloriaDataContainer* data ) ;
    
    
    virtual void check( GloriaDataContainer* data ) ; 
    
    
    /*! Called after data processing for clean up.
     */
    virtual void end() ;
    
    
  protected:

    /**Test method for const.*/
    void printEndMessage() const ;

    //! Name of the image collection containing images to be added
    string _inputCollectionName;

    //! Name of the output image collection
    string _outputCollectionName;

    //! Flag the output collection to be permanent
    bool _permanentCollection;

    //! Keywords which should match for considered frames
    std::vector<std::string> _matchKeywords;

    int _nLoops ;
    int _nInputImages ;
    int _nOutputImages ;

    GloriaFitsImage *_medianImage;

  } ;
  
} // end namespace luiza
#endif



