// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef SimpleSourceFinder_h
#define SimpleSourceFinder_h 1

#include <Python.h>
#include <numpy/arrayobject.h>
#include "luiza/Processor.h"

using namespace gloria;

namespace luiza {
  
  
  //! Simple processor to find stars centres on the frame
  /*! Processor to find objects (stars) on images in the input collection.
   *  Simple algorithm based on Python library is used
   *
   *  \par Input 
   *  Processor takes images from Gloria data collection
   *
   *  \param InputCollectionName -
   *             Name of the collection containing images to be added
   *
   *  \par Output
   *
   *  Object lists are stored in the output collection
   *
   *  \param OutputCollectionName - 
   *            Name of the collection where object table will be stored
   * 
   *  \param PermanentCollection  -
   *   Flag output collection, as permanent, not deleted after loop is
   *   finished (default is false)
   *
   */

  class SimpleSourceFinder : public Processor {
    
  public:
    
    virtual Processor*  newProcessor() { return new SimpleSourceFinder ; }
    
    
    SimpleSourceFinder() ;
    
    /*! Called at the begin of the job before anything is read.
     * Use to initialize the processor, e.g. book histograms.
     */
    virtual void init() ;
    
    /*! Called for every data loop
     */
    virtual void processData( GloriaDataContainer* data ) ;
    
    
    virtual void check( GloriaDataContainer* data ) ; 
    
    
    /*! Called after data processing for clean up.
     */
    virtual void end() ;
    
    
  protected:

    /**Test method for const.*/
    void printEndMessage() const ;

    //! Name of the image collection containing images to be added
    string _inputCollectionName;

    //! Name of the output image collection
    string _outputCollectionName;

    //! Flag the output collection to be permanent
    bool _permanentCollection;

    int _nLoops ;
    int _nInputImages ;
    int _nOutputLists ;

  } ;
  
} // end namespace luiza
#endif



