# python module for SimpleSourceFinder processor
import numpy as np
import mahotas.labeled

def find_sources(frame):
	threshold = 5

	# find proper median and stddev of image (recurrent)
	bg = np.median(frame)
	dbg = threshold*np.std(frame)
	for i in xrange(3):
		bg = np.median(frame[frame<bg+dbg])
		dbg = threshold*np.std(frame[frame<bg+dbg])
	
	binimage = frame>(bg+dbg)

	labels, nr_clusters = mahotas.labeled.label(binimage)

	star_positions = mahotas.center_of_mass(frame, labels)

	# without the line below, labeled_sum does not work
#	labels = labels/1
	l1 = np.array(labels, dtype=np.int32)
	star_masses = mahotas.labeled.labeled_sum(frame-bg, l1)

	# merge positions and cluster signal matrices
	stars = np.hstack((star_positions[:,2::-1]+(1,1), [[el] for el in star_masses]))
	# sort by cluster signal and ommit first element
	# first element is sum of signal outside star clusters
	stars = np.array(stars[stars[:,2].argsort()[::-1]])[1:]
	
	return stars
