// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef ExampleProcessor_h
#define ExampleProcessor_h 1

#include "luiza/Processor.h"

using namespace gloria;

namespace luiza {
  
  
  //! Example processor for users. 
  /*! Processor example,
   *  based on luiza::TestProcessor.
   *  
   *  Checks sizes, average pixel values and RMS of all images stored
   *  in given image collection (or in all collections within
   *  GloriaDataContainer if collection name not given).
   *
   *  Prints table with all details at the end of the processing.
   *  If output file name given, table is written to text data file.
   *
   *  \param ImageCollectionName -
   *             Name of the collection containing images to be added
   *
   *  \param WindowXY - 
   *             Window for average and RMS calculation given as
   *             vector of four numbers:  X1 Y1 X2 Y2 (from 1 to size
   *             each)
   *
   *  \param OutputTextFile - name of the output text file where table
   *              with image statistics should be written to.
   *
   */

  class ExampleProcessor : public Processor {
    
  public:
    
    virtual Processor*  newProcessor() { return new ExampleProcessor ; }
    
    
    ExampleProcessor() ;
    
    //! Called at the begin of the job before anything is read.
    /*! Use to initialize the processor.
     */
    virtual void init() ;
    
    //! Called before first data loop.
    /*! Can be used to create permanent data structures, if needed.
     */
    virtual void startAnalysis( GloriaDataContainer* ) ;
    
    //! Called for every data collection
    
    virtual void processData( GloriaDataContainer* data ) ;
    
    
    virtual void check( GloriaDataContainer* data ) ; 
    
    //! Called after all data processing loops to finalize the analysis

    virtual void concludeAnalysis( GloriaDataContainer* ) ;
    
    //! Called after data processing for clean up.

    virtual void end() ;
    
    
  protected:

    /**Example method for const.*/
    void printEndMessage() const ;

    string _imageCollectionName;

    string _outputTextFile;

    // Vector for optional window definition

    std::vector<int > _WindowXY;

    int _Xmin,_Xmax,_Ymin,_Ymax;

    int _nLoops ;
    int _nImgCollections ;
    int _nImages ;

    GloriaFitsTable *_imgTable;

  } ;
  
} // end namespace luiza
#endif



