// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef RootImageViewer_h
#define RootImageViewer_h 1

#include "RootProcessor.h"


// root includes

#include "TASImage.h"
#include "TH2D.h"
#include "TGraph.h"
#include "TTimer.h"

using namespace gloria;

namespace luiza {
  
  
  //! Processor allowing to view images during processing
  /*! Processor based on CERN root classes
   *  Shows images stored in given collection as root TASImage
   *
   *  \param  ImageCollectionName -
   *             Name of the collection containing images to be shown
   *
   */

  class RootImageViewer : public RootProcessor {
    
  public:
    
    virtual Processor*  newProcessor() { return new RootImageViewer ; }
    
    
    RootImageViewer() ;
    
    /** Called at the begin of the job before anything is read.
     * Use to initialize the processor, e.g. book histograms.
     */
    virtual void init() ;
    
    /** Called for every data collection
     */
    virtual void processData( GloriaDataContainer* data ) ;
    
    
    virtual void check( GloriaDataContainer* data ) ; 
    
    
    /** Called after data processing for clean up.
     */
    virtual void end() ;
    
    
  protected:

    void printEndMessage() const ;

    string _imageCollectionName;

    string _listCollectionName;

    string _canvasName;

    string _canvasTitle;

    bool _showHistogram;

    static TTimer* rootTimer;

    TASImage* rootImage;
    TH2D* rootHistogram;
    TGraph* rootGraph;

    int _pauseTime;
    std::vector<int > _CanvasGeom;

    int _nLoops ;
    int _nImages ;

  } ;
  
} // end namespace luiza
#endif



