// -*- mode: c++; mode: auto-fill; mode: flyspell-prog; -*-

#ifndef SatelliteSearching_h
#define SatelliteSearching_h

#include "luiza/Processor.h"
#include <sstream>
#include "luiza/VerbosityLevels.h"
#include "luiza/LuizaExceptions.h"
#include "streamlog/streamlog.h"


////
//#include "gloria/GloriaFitsImage.h"
//#include "gloria/GloriaAstrometry.h"
#include <list>
#include <set>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <cmath>
#include <string>
////

using namespace gloria;
using namespace std;
class satellite;

const float sigmaTrigger = 3;
const long minLength = 10;

extern double median,trigger; 
extern long *pixel, X, Y;
extern vector< satellite > satellites;
extern set<long> checked;
extern satellite s;
extern double p[16];
/*
	p[0] = path length;    
	p[1] = X1;
	p[2] = Y1;
	p[3] = X2;
	p[4] = Y2;
	p[5] = a;
	p[6] = b;
	p[7] = fitting arguments;
	p[8] = angle;
	p[9] = sigma;
	p[10] = A;
	p[11] = C;
	p[12] = x;
	p[13] = y;
	p[14] = dx;
	p[15] = dy;
*/

namespace luiza 
{
  //! Example processor for users. 
  /*! Processor example,
   *  based on luiza::TestProcessor.
   *  
   *  Checks sizes, average pixel values and RMS of all images stored
   *  in given image collection (or in all collections within
   *  GloriaDataContainer if collection name not given).
   *
   *  Prints table with all details at the end of the processing.
   *  If output file name given, table is written to text data file.
   *
   *  \param ImageCollectionName -
   *             Name of the collection containing images to be added
   *
   *  \param WindowXY - 
   *             Window for average and RMS calculation given as
   *             vector of four numbers:  X1 Y1 X2 Y2 (from 1 to size
   *             each)
   *
   *  \param OutputTextFile - name of the output text file where table
   *              with image statistics should be written to.
   *
   */

  	class SatelliteSearching : public Processor
	{
      	public:
    		virtual Processor*  newProcessor() { return new SatelliteSearching ; }
    
   		SatelliteSearching() ;
    
    		//! Called at the begin of the job before anything is read.
    		/*! Use to initialize the processor.
     		*/
    		virtual void init() ;
    
    		//! Called before first data loop.
    		/*! Can be used to create permanent data structures, if needed.
    		 */
    		virtual void startAnalysis( GloriaDataContainer* ) ;
    
    		//! Called for every data collection
    		virtual void processData( GloriaDataContainer* data ) ;
    
    
    		virtual void check( GloriaDataContainer* data ) ;  

    		//! Called after all data processing loops to finalize the analysis
    		virtual void concludeAnalysis( GloriaDataContainer* ) ;
    
    		//! Called after data processing for clean up.
    		virtual void end() ;
    	
  	protected:
    		/**Example method for const.*/
    		void printEndMessage() const ;

    		string _imageCollectionName;
    		string _outputTextFile;

    		//int _Xmin,_Xmax,_Ymin,_Ymax;
    		int _nLoops ;
    		int _nImgCollections ;
    		int _nImages ;
		GloriaFitsTable *_imgTable;

		void MedianValue();
		void EraseSatellite();
		void WriteResults();
		double quantity;
  	};
  
} // end namespace luiza
#endif


class satellite
{
public:
	long start;
	long end;
	long length;
	list<long> path;
};
