#include "SatelliteSearching.h"
#include "FirstFit.h"


void FirstFit::Start()
{
	sigma = 1.;
	A = 200;
	C = 700;
	lambda = 0.01;
	k = 2;

	chi2Old = 1.e300;
	chi2 = 1.e299;

	p[8] = atan2(satellites.begin()->end/X-satellites.begin()->start/X,satellites.begin()->end%X-satellites.begin()->start%X) - M_PI/4; 
	
	FitLine();
	CalculatePerpendicularData();
	FitPerpendicularGauss();
}

void FirstFit::FitLine()
{
	double Sx=0, Sy=0, Sxy=0, Sxx=0, n =0, a=0, b=0;
	long X1,Y1,X2,Y2;
	double x,y;

	list<long>::iterator it;	
  	for(it=satellites.begin()->path.begin(); it!=satellites.begin()->path.end(); ++it)
	{
		if(it==satellites.begin()->path.begin())
		{
			X1 = (*it)%X;
			Y1 = (*it)/X;
		}

		x = ((*it)%X-X1)*cos(p[8]) + ((*it)/X-Y1)*sin(p[8]);
		y = ((*it)/X-Y1)*cos(p[8]) - ((*it)%X-X1)*sin(p[8]);

    		Sx += x;
		Sy += y;
		Sxy += x*y;
		Sxx += x*x;
		++n;
	}
	--it;
	X2 = (*it)%X;
	Y2 = (*it)/X;

	a = (n*Sxy-Sx*Sy)/(n*Sxx-Sx*Sx);
	b = (Sxx*Sy-Sx*Sxy)/(n*Sxx-Sx*Sx);

	p[0] = n;
	p[1] = X1;
	p[2] = Y1;
	p[3] = X2;
	p[4] = Y2;
	p[5] = a;
	p[6] = b;
}	

void FirstFit::FitPerpendicularGauss()
{
	double D[(int)p[7]];
	double Z[(int)p[7]][3];
	double ZtZ[3][3];
	double ZtZ_1[3][3];
	double ZtD[3];

	while(fabs((chi2 - chi2Old)/chi2Old) > 1e-5 || chi2 == chi2Old)
	{	
		CalculateInit(D,Z);
		CalculateZtZ(ZtZ,Z);
		CalculateZtZ_1(ZtZ,ZtZ_1);
		CalculateZtD(ZtD,D,Z);
		CalculateFinal(ZtZ_1,ZtD,D);	
	}

	p[9] = sigma;
	p[10] = A;
	p[11] = C;
}

void FirstFit::CalculatePerpendicularData()
{
	ofstream plik("Calculations/perpendicular.dat");
	long r=0;
	long X1 = p[1], X2 = p[3];
	long Y1 = p[2], Y2 = p[4];
	double x,y;

	for(long i=X1-4; i<=X2+4; ++i)
	for(long j=((p[2] < p[4]) ? p[2] : p[4])-4; j<=((p[2] < p[4]) ? p[4] : p[2]+4); ++j)
	{
		x = (i-X1)*cos(p[8]) + (j-Y1)*sin(p[8]);
		y = (j-Y1)*cos(p[8]) - (i-X1)*sin(p[8]);

		if(ConditionInsidePath(x,y))
		{
			plik << (-p[5]*x+y-p[6])/sqrt(p[5]*p[5]+1) << " " << pixel[j*X+i] << endl;		
			++r;
		}
	}

	plik.close();
	p[7] = r;
}

bool FirstFit::ConditionInsidePath(double x,double y)
{
	return (x+(y-p[6])*p[5])/(p[5]*p[5]+1) > 1 && (x+(y-p[6])*p[5])/(p[5]*p[5]+1) < (p[3]-p[1])*cos(p[8]) + (p[4]-p[2])*sin(p[8]) - 1;
}

double FirstFit::f(double x)
{
	return A/(sigma*sqrt(2*M_PI))*exp(-x*x/(2*sigma*sigma)) + C;	
}
	
double FirstFit::df_sigma(double x)
{
	return (f(x)-C)*(-1/sigma + x*x/pow(sigma,3));
}
	
double FirstFit::df_A(double x)
{
	return 1/(sigma*sqrt(2*M_PI))*exp(-x*x/(2*sigma*sigma));
}

double FirstFit::df_C(double x)
{
	return 1;
}

void FirstFit::CalculateInit(double D[], double Z[][3])
{
	ifstream plik("Calculations/perpendicular.dat");
	double x,y;
	
	for(int i=0; i<(int)p[7]; ++i)
	{
		plik >> x >> y;
		D[i] = y - f(x);
		Z[i][0] = df_sigma(x);
		Z[i][1] = df_A(x);
		Z[i][2] = df_C(x);
	}

	plik.close();
}

void FirstFit::CalculateZtZ(double ZtZ[][3], double Z[][3])
{
 	for(int i=0; i<3; ++i)
	for(int j=0; j<3; ++j)
		ZtZ[i][j] = 0;
		
	
  	for(int i=0; i<3; ++i)
	for(int j=0; j<3; ++j)
	for(int k=0; k<(int)p[7]; ++k)	
		ZtZ[i][j] += Z[k][i] * Z[k][j];

	for(int i=0; i<3; ++i)
		ZtZ[i][i] += lambda*ZtZ[i][i];
}

void FirstFit::CalculateZtD(double ZtD[], double D[], double Z[][3])
{	
	for(int j=0; j<3; ++j)
    		ZtD[j] = 0;

	double chi2 = 0;
	for(int i=0; i<p[7]; ++i)
	for(int j=0; j<3; ++j)
    			ZtD[j] += Z[i][j]*D[i];
}

void FirstFit::CalculateFinal(double ZtZ_1[][3], double ZtD[], double D[])
{
	if(chi2 == -1) chi2 = Chi2(D);
	chi2Old = chi2;

	for(int i=0; i<3; ++i)
	{
		sigma += ZtZ_1[0][i]*ZtD[i];
		A += ZtZ_1[1][i]*ZtD[i];
		C += ZtZ_1[2][i]*ZtD[i];
	}

	if(chi2Old < Chi2(D))
	{
		for(int i=0; i<3; ++i)
		{
			sigma -= ZtZ_1[0][i]*ZtD[i];
			A -= ZtZ_1[1][i]*ZtD[i];
			C -= ZtZ_1[2][i]*ZtD[i];
		}
		
		lambda *= k;
	}
	else
	{
		chi2 = Chi2(D);
		lambda /= k;
	}

	
}

void FirstFit::CalculateZtZ_1(double ZtZ[][3], double ZtZ_1[][3])
{
	for(int i=0; i<3; ++i)
	for(int j=0; j<3; ++j)
		ZtZ_1[i][j] = 0;

	// matrix LU (written in the same matrix)
	double suma;
	int k=0;

	for(int j=0; j<3; j++)
	for(int i=0; i<3; i++)
	{
		k=0;
		suma=0;
		while(k<i && k<j)
		{
			suma += ZtZ[i][k]*ZtZ[k][j];
			k++;
		}

		ZtZ[i][j] -= suma;

		if(i>j)
			ZtZ[i][j] /= ZtZ[j][j];
	}

	// Inverse matrix
	double wektor[3];

	for(int a=0; a<3; a++)
	{
	
		for(int i=0; i<3; i++)
			wektor[i] = 0;
		wektor[a]=1;
	
		for(int i=0; i<3; i++)
		{
			suma=0;
			k=0;
			while(k<i)
			{	
				suma += ZtZ[i][k]*wektor[k]; 
				k++;
			}
			wektor[i] -= suma;
		}

		for(int i=3-1; i>-1; i--)
		{
			suma=0;
			k=3-1;
			while(k>i)
			{	
				suma += ZtZ[i][k]*wektor[k]; 
				k--;
			}
			wektor[i] -= suma;
			wektor[i] /= ZtZ[i][i];		
		}

		for(int i=0; i<3; i++)
			ZtZ_1[i][a] = wektor[i];
	}
}

double FirstFit::Chi2(double D[])
{
	ifstream plik("Calculations/perpendicular.dat");
	double x,y;
	double chi2 = 0;

	for(int i=0; i<(int)p[7]; ++i)
	{
		plik >> x >> y;
		D[i] = y - f(x);
		chi2 += pow(D[i],2);
	}

	plik.close();

	return chi2;	
}

double FirstFit::sigma, FirstFit::A, FirstFit::C, FirstFit::chi2, FirstFit::chi2Old, FirstFit::lambda, FirstFit::k;




