#include "SatelliteSearching.h"
#include "Fit2DGauss.h"
	

void Fit2DGauss::Start()
{
	chi2Old = 	1.e100;
	chi2 	=	1.e50;
	lambda 	= 	0.01;
	k 	= 	2;

	X1 	= 	0;
	X2 	= 	(p[3]-p[1])*cos(p[8]) + (p[4]-p[2])*sin(p[8]);
	a 	= 	p[5];
	b 	= 	p[6];
	sigma 	= 	p[9];
	A 	= 	p[10];
	C 	= 	p[11];
	
	CalculateDataRotated();
	FitSatellite();

	if(sigma>0 && X1<X2 && A>0 && sqrt(pow(X1-X2,2)+pow(a*(X1-X2),2))>minLength)		
	{
		CalculateData();
		SaveResults();
		//PlotSatellite();
		EraseTrace();
	}
}	

void Fit2DGauss::FitSatellite()
{
	double D[(int)p[7]];
	double Z[(int)p[7]][7];
	double ZtZ[7][7];
	double ZtZ_1[7][7];
	double ZtD[7];
	double G[(int)p[7]];

	while(fabs((chi2 - chi2Old)/chi2Old) > 1e-7 && chi2 != chi2Old)
	{
		CalculateInit(D,Z,G);
		CalculateZtZ(ZtZ,Z,G);
		CalculateZtZ_1(ZtZ,ZtZ_1);
		CalculateZtD(ZtD,D,Z,G);
		CalculateFinal(ZtZ_1,ZtD,D,G);	
	}

	for(int i=0; i<7; ++i)
		errors[i] = sqrt(ZtZ_1[i][i]);

	//cout << sqrt(ZtZ_1[3][3]) << " " << sqrt(ZtZ_1[4][4]) << " " << Chi2(D,G)/(p[7]-8)<<  endl;	
}

void Fit2DGauss::EraseTrace()
{
	X1 = p[1];
	X2 = p[3];
	a = p[5];
	b = p[6];	

	long 	Y1 = (p[2] < p[4]) ? p[2] : p[4],
		Y2 = (p[2] < p[4]) ? p[4] : p[2];

	for(long i=X1-5; i<=X2+5; ++i)
	for(long j=Y1-5; j<=Y2+5; ++j)
	{
		pixel[j*X+i] -= f(i,j) - C;
		checked.insert(j*X+i);
	}	
}


void Fit2DGauss::PlotSatellite()
{
	ofstream data("Calculations/data.dat");
	list<long>::iterator it;
	for(it=satellites.begin()->path.begin(); it!=satellites.begin()->path.end(); ++it)
		data << (*it)%X << " " << (*it)/X << " " << pixel[(*it)] << endl;
	data.close();

	ofstream plot("Plot/plot3.dat");
	plot 	<< "PI = " << M_PI << endl
		<< "sigma = " << p[9] << endl
	 	<< "A = " << p[10] << endl
		<< "C = " << p[11] << endl
	 	<< "X1 = " << p[1] << endl
		<< "X2 = " << p[3] << endl
	 	<< "a = " << p[5] << endl
		<< "b = " << p[6] << endl
		<< "S = A/(sigma**(2)*2*PI)" << endl
		<< "T = -1/(2*sigma**(2))" << endl

		<< "r(x,y) = (x+a*(y-b))/(a*a+1)" << endl
		<< "f(x,y) = S*exp(T*((x-X1)**2+(y-a*X1-b)**2))+C" << endl
		<< "g(x,y) = S*exp(T*((x-X2)**2+(y-a*X2-b)**2))+C" << endl
		<< "h(x,y) = S*exp(T*(((-a*x+y-b)/sqrt(a*a+1))**2))+C" << endl 

		<< "R = " << pow(pow(p[1]-p[3],2) + pow(p[5]*(p[1]-p[3]),2),0.5)   << endl 
		<< "Rx1 = (X1+X2)/2.-R" << endl
		<< "Rx2 = (X1+X2)/2.+R" << endl
		<< "Ry1 = (a*(X1+X2)+2*b)/2.-R" << endl
		<< "Ry2 = (a*(X1+X2)+2*b)/2.+R" << endl
		<< "set xrange[Rx1:Rx2]" << endl
		<< "set yrange[Ry1:Ry2]" << endl
		<< "set xlabel \"x\"" << endl
		<< "set ylabel \"y\"" << endl
		<< "unset key" << endl
 
		//<< "set hidden3d" << endl
		<< "set isosamples " << ((fabs(X1-X2) > fabs(a*(X1-X2))) ? 2*fabs(X1-X2)+5 : 2*fabs(a*(X1-X2))+5) << endl
		<< "set samples 50" << endl
		<< "set pm3d" << endl
		<< "set palette negative defined (-6 '#D53E4F',0 '#D53E4F',1 '#F46D43',2 '#FDAE61',3 '#FEE08B',4 '#E6F598',5 '#ABDDA4',6 '#66C2A5',7 '#3288BD',9 '#3288BD' )" << endl

		<< "splot \"Calculations/2DGaussData.dat\" u 1:2:3, (r(x,y)<X1) ? f(x,y) : ((r(x,y)>X2) ? g(x,y) : h(x,y)) lc rgb '#CECEF6', \"Calculations/data.dat\" u 1:2:(C) with line lw 3 lc rgb \"blue\", \"Calculations/data.dat\" u 1:2:3 with line lw 3 lc rgb \"red\"" << endl
		//<< "splot \"Calculations/2DGaussData.dat\" u 1:2:3, (r(x,y)<X1) ? f(x,y) : ((r(x,y)>X2) ? g(x,y) : h(x,y)) lc rgb '#CECEF6'" << endl		
		<< "pause -1" << endl;
	plot.close();

	
	system("gnuplot -p Plot/plot3.dat");
}

void Fit2DGauss::SaveResults()
{
	double x1,y1,x2,y2;
	
	x1 = (X1)*cos(-p[8]) + (a*X1+b)*sin(-p[8]) + p[1];
	y1 = (a*X1+b)*cos(-p[8]) - (X1)*sin(-p[8]) + p[2];

	x2 = (X2)*cos(-p[8]) + (a*X2+b)*sin(-p[8]) + p[1];
	y2 = (a*X2+b)*cos(-p[8]) - (X2)*sin(-p[8]) + p[2];

	p[0] = sqrt(pow(x2-x1,2)+pow(y2-y1,2));
	p[1] = x1;
	p[2] = y1;
	p[3] = x2;
	p[4] = y2;

	if(x2==x1)
	 	cout << "vertical\n";
	else
	{
 		p[5] = (y2-y1)/(x2-x1);
		p[6] = y1 - x1*p[5];
	}

	p[9] = sigma;
	p[10] = A;
	p[11] = C;

	p[12] = (x1+x2)/2.;
	p[13] = (y1+y2)/2.;

	p[14] = 0.5*sqrt(pow((cos(-p[8])+a*sin(-p[8]))*errors[3],2)+pow((X1*sin(-p[8]))*errors[5],2)+pow(sin(-p[8])*errors[6],2)+pow((cos(-p[8])+a*sin(-p[8]))*errors[4],2)+pow((X2*sin(-p[8]))*errors[5],2)+pow(sin(-p[8])*errors[6],2));
	p[15] = 0.5*sqrt(pow((a*cos(-p[8])-sin(-p[8]))*errors[3],2)+pow((X1*cos(-p[8]))*errors[5],2)+pow(cos(-p[8])*errors[6],2)+pow((a*cos(-p[8])-sin(-p[8]))*errors[4],2)+pow((X2*cos(-p[8]))*errors[5],2)+pow(cos(-p[8])*errors[6],2));
}


void Fit2DGauss::CalculateDataRotated()
{
	ofstream plik("Calculations/2DGaussData.dat");
	long r=0;
	long X1 = p[1], X2 = p[3];
	long Y1 = p[2], Y2 = p[4];


	for(long i=X1-30; i<=X2+30; ++i)
	for(long j=((p[2] < p[4]) ? p[2] : p[4])-30; j<=((p[2] < p[4]) ? p[4] : p[2])+30; ++j)
	{
		plik << (i-X1)*cos(p[8])+(j-Y1)*sin(p[8]) << " " << -(i-X1)*sin(p[8])+(j-Y1)*cos(p[8]) << " " << pixel[j*X+i] << endl;		
		++r;
	}

	plik.close();
	p[7] = r;
}

void Fit2DGauss::CalculateData()
{
	ofstream plik("Calculations/2DGaussData.dat");
	long r=0;
	long X1 = p[1], X2 = p[3];
	long Y1 = p[2], Y2 = p[4];


	for(long i=X1-30; i<=X2+30; ++i)
	for(long j=((p[2] < p[4]) ? p[2] : p[4])-30; j<=((p[2] < p[4]) ? p[4] : p[2])+30; ++j)
	{
		plik << i << " " << j << " " << pixel[j*X+i] << endl;		
		++r;
	}

	plik.close();
	p[7] = r;
}

int Fit2DGauss::Region(double x, double y)
{
	double rx = (x+a*(y-b))/(a*a+1);

	return (rx<X1) ? 1: ((rx>X2) ? 3:2);
}

double Fit2DGauss::f(double x, double y)
{
	double wynik = 0;

	switch(Region(x,y))
	{
	case 1:	wynik = pow(x-X1,2)+pow(y-a*X1-b,2);	break;
	case 2:	wynik = pow((-a*x+y-b)/sqrt(a*a+1),2);	break;
	case 3:	wynik = pow(x-X2,2)+pow(y-a*X2-b,2);	break;
	}

	return A/(pow(sigma,2)*2*M_PI)*exp(-1/(2*sigma*sigma)*wynik) + C;	
}
	
double Fit2DGauss::df_sigma(double x, double y)
{
	double wynik = 0;

	switch(Region(x,y))
	{
	case 1:	wynik = pow(x-X1,2)+pow(y-a*X1-b,2);	break;
	case 2:	wynik = pow((-a*x+y-b)/sqrt(a*a+1),2);	break;
	case 3:	wynik = pow(x-X2,2)+pow(y-a*X2-b,2);	break;
	}

	return (f(x,y)-C)*(-2/sigma + wynik/pow(sigma,3));
}
	
double Fit2DGauss::df_A(double x, double y)
{
	return (f(x,y)-C)/A;
}

double Fit2DGauss::df_C(double x, double y)
{
	return 1;
}

double Fit2DGauss::df_X1(double x, double y)
{
	double wynik = 0;

	switch(Region(x,y))
	{
	case 1:	wynik = -x+X1-(y-a*X1-b)*a;	break;
	case 2:	wynik = 0;			break;
	case 3:	wynik = 0;			break;
	}

	return wynik*(f(x,y)-C)*(-1/(sigma*sigma));
}

double Fit2DGauss::df_X2(double x, double y)
{
	double wynik = 0;

	switch(Region(x,y))
	{
	case 1:	wynik = 0;			break;
	case 2:	wynik = 0;			break;
	case 3:	wynik = -x+X2-(y-a*X2-b)*a;	break;
	}

	return wynik*(f(x,y)-C)*(-1/(sigma*sigma));
}

double Fit2DGauss::df_a(double x, double y)
{
	double wynik = 0;

	switch(Region(x,y))
	{
	case 1:	wynik = (a*X1+b-y)*X1;				break;
	case 2:	wynik = (a*x+b-y)*(a*(y-b)+x)/pow(a*a+1,2);	break;
	case 3:	wynik = (a*X2+b-y)*X2;				break;
	}

	return wynik*(f(x,y)-C)*(-1/(sigma*sigma));
}

double Fit2DGauss::df_b(double x, double y)
{
	double wynik = 0;

	switch(Region(x,y))
	{
	case 1:	wynik = a*X1+b-y;		break;
	case 2:	wynik = (a*x-y+b)/(a*a+1);	break;
	case 3:	wynik = a*X2+b-y;		break;
	}

	return wynik*(f(x,y)-C)*(-1/(sigma*sigma));
}


void Fit2DGauss::CalculateInit(double D[], double Z[][7], double G[])
{
	ifstream plik("Calculations/2DGaussData.dat");
	double x,y,z;
	
	for(int i=0; i<(int)p[7]; ++i)
	{
		plik >> x >> y >> z;
		D[i] = z - f(x,y);
		G[i] = 1/z;
		Z[i][0] = df_sigma(x,y);
		Z[i][1] = df_A(x,y);
		Z[i][2] = df_C(x,y);
		Z[i][3] = df_X1(x,y);
		Z[i][4] = df_X2(x,y);
		Z[i][5] = df_a(x,y);
		Z[i][6] = df_b(x,y);
	}

	plik.close();
}

void Fit2DGauss::CalculateZtZ(double ZtZ[][7], double Z[][7], double G[])
{
 	for(int i=0; i<7; ++i)
	for(int j=0; j<7; ++j)
		ZtZ[i][j] = 0;
		
	
  	for(int i=0; i<7; ++i)
	for(int j=0; j<7; ++j)
	for(int k=0; k<(int)p[7]; ++k)	
		ZtZ[i][j] += Z[k][i] * G[k] * Z[k][j];

	for(int i=0; i<7; ++i)
		ZtZ[i][i] += lambda*ZtZ[i][i];
}

void Fit2DGauss::CalculateZtD(double ZtD[], double D[], double Z[][7], double G[])
{	
	for(int j=0; j<7; ++j)
    		ZtD[j] = 0;

	double chi2 = 0;
	for(int i=0; i<p[7]; ++i)
	for(int j=0; j<7; ++j)
    			ZtD[j] += Z[i][j] * G[i] * D[i];
}

void Fit2DGauss::CalculateFinal(double ZtZ_1[][7], double ZtD[], double D[], double G[])
{
	if(chi2 == -1) chi2 = Chi2(D,G);
	chi2Old = chi2;

	for(int i=0; i<7; ++i)
	{
		sigma += ZtZ_1[0][i]*ZtD[i];
		A += ZtZ_1[1][i]*ZtD[i];
		C += ZtZ_1[2][i]*ZtD[i];
		X1 += ZtZ_1[3][i]*ZtD[i];
		X2 += ZtZ_1[4][i]*ZtD[i];
		a += ZtZ_1[5][i]*ZtD[i];
		b += ZtZ_1[6][i]*ZtD[i];
	}

	if(chi2Old < Chi2(D,G))
	{
		for(int i=0; i<7; ++i)
		{
			sigma -= ZtZ_1[0][i]*ZtD[i];
			A -= ZtZ_1[1][i]*ZtD[i];
			C -= ZtZ_1[2][i]*ZtD[i];
			X1 -= ZtZ_1[3][i]*ZtD[i];
			X2 -= ZtZ_1[4][i]*ZtD[i];
			a -= ZtZ_1[5][i]*ZtD[i];
			b -= ZtZ_1[6][i]*ZtD[i];
		}
		
		lambda *= k;
	}
	else
	{
		chi2 = Chi2(D,G);
		lambda /= k;
	}	
}


void Fit2DGauss::CalculateZtZ_1(double ZtZ[][7], double ZtZ_1[][7])
{
	for(int i=0; i<7; ++i)
	for(int j=0; j<7; ++j)
		ZtZ_1[i][j] = 0;

	// macierz LU (w tej samej macierzy)
	double suma;
	int k=0;

	for(int j=0; j<7; j++)
	for(int i=0; i<7; i++)
	{
		k=0;
		suma=0;
		while(k<i && k<j)
		{
			suma += ZtZ[i][k]*ZtZ[k][j];
			k++;
		}

		ZtZ[i][j] -= suma;

		if(i>j)
			ZtZ[i][j] /= ZtZ[j][j];
	}

	// Macierz odwrotna
	double wektor[7];

	for(int a=0; a<7; a++)
	{
	
		for(int i=0; i<7; i++)
			wektor[i] = 0;
		wektor[a]=1;
	
		for(int i=0; i<7; i++)
		{
			suma=0;
			k=0;
			while(k<i)
			{	
				suma += ZtZ[i][k]*wektor[k]; 
				k++;
			}
			wektor[i] -= suma;
		}

		for(int i=7-1; i>-1; i--)
		{
			suma=0;
			k=7-1;
			while(k>i)
			{	
				suma += ZtZ[i][k]*wektor[k]; 
				k--;
			}
			wektor[i] -= suma;
			wektor[i] /= ZtZ[i][i];		
		}

		for(int i=0; i<7; i++)
			ZtZ_1[i][a] = wektor[i];
	}
}

double Fit2DGauss::Chi2(double D[], double G[])
{
	ifstream plik("Calculations/2DGaussData.dat");
	double x,y,z;
	double chi2 = 0;

	for(int i=0; i<(int)p[7]; ++i)
	{
		plik >> x >> y >> z;
		D[i] = z - f(x,y);
		G[i] = 1/z;
		chi2 += pow(D[i]*G[i],2);
	}

	plik.close();

	return chi2;	
}

double Fit2DGauss::sigma, Fit2DGauss::A, Fit2DGauss::C, Fit2DGauss::X1, Fit2DGauss::X2, Fit2DGauss::a, Fit2DGauss::b, Fit2DGauss::chi2, Fit2DGauss::chi2Old, Fit2DGauss::lambda, Fit2DGauss::k, Fit2DGauss::errors[7];



