#include "SatelliteSearching.h"
#include "SearchSatellite.h"


void SearchSatellite::Start()
{	
}

long SearchSatellite::Neighbour(long x)
{
	switch(x)
	{
	case 0:	return X-1; 	break;
	case 1:	return X;	break;
	case 2: return X+1;	break; 
	case 3: return 1;	break;
	case 4:	return -X+1;	break;
	case 5:	return -X;	break; 
	case 6: return -X-1;	break;
	case 7: return -1;	break;
	case 8: return 0;	break;
	}
}


bool SearchSatellite::NotEdge(long i)
{
	int width = 40;
	bool notEdge = true;

	if(i < width*X || i >= X*(Y-width) || i%X < width || (i+width)%X < width) notEdge = false;

	return notEdge;
}

bool SearchSatellite::ConditionMedian(long position, float constant)
{
	return median+constant*trigger < pixel[position];
}

float SearchSatellite::ConditionMedianProximity(long i, float constant)
{
	long width = 10;
	vector<long> pixelV(pow(2*width+1,2));
	int n=0;
			
	for(long x=i%X-width; x<=i%X+width; ++x)
	for(long y=i/X-width; y<=i/X+width; ++y)
	{
		pixelV[n] = pixel[y*X+x];
		++n;
	}
			
	sort(pixelV.begin(), pixelV.end());
			
	return pixelV[0.5*pow(2*width+1,2)] + constant*(pixelV[0.83*pow(2*width+1,2)] - pixelV[0.17*pow(2*width+1,2)])/2;
}

bool SearchSatellite::ConditionLine(float constant, long start, long position, long length)
{
	return sqrt(pow(position%X - start%X,2)+pow(position/X - start/X,2)) >= constant*(length-1);
}

bool SearchSatellite::ConditionDirect(long positionOld, long position)
{
	return positionOld != position;
}


long SearchSatellite::SearchPath(long i)
{
	long value = 0;
	long position = i;
	long direction = 0;
	long directionOld = 8;
	long positionOld = -1;
	long length = 0;

	s.start = i;
	s.path.clear();
	
	while(pixel[position] > ConditionMedianProximity(i,sigmaTrigger) && ConditionMedian(position,sigmaTrigger) && ConditionDirect(positionOld,position) && ConditionLine(0.7,i,position,length) && NotEdge(position)) 
	{
		s.path.push_back(position);
		
		for(int j=1; j<6; ++j)
		if(NotEdge(position+Neighbour(j)) && pixel[position+Neighbour(j)] > value && j != directionOld && j != (directionOld+1)%8 && j != (directionOld-1)%8) 
		{
			value = pixel[position+Neighbour(j)];
			direction = j;
		}
		
		directionOld = (direction+4)%8;		
		positionOld = position;
		position += Neighbour(direction);
		value = 0;
		++length;
		direction = 8;
	}

	s.end = positionOld;
	s.length;	
	
	return length;	
}


void SearchSatellite::FindPossibleSatellites()
{	
	checked.clear();
	int counter = 0;

	for(long i=0; i<X*Y; ++i)
	if(NotEdge(i) && ConditionMedian(i,sigmaTrigger) && pixel[i] > ConditionMedianProximity(i,sigmaTrigger))
	{
		counter = 0;
		for(int j=0; j<8; ++j)
			if(pixel[i+Neighbour(j)] > pixel[i])
				++counter;			
		if(counter == 2)
			if(SearchPath(i) > minLength)
				satellites.push_back(s);
	}

	sort(satellites.begin(), satellites.end(), [](const satellite &a, const satellite &b){return a.length > b.length;});
		
}

